//  //
//  ###  ##### #   #               -= Confix Library =-                  //
// #   # #     ## ## ConfigFile.cpp - Configuration file                 //
// #     ####   ###                                                      //
// #     #      ###  Representation of a configuration file              //
// #   # #     ## ##                                                     //
//  ###  #     #   # R1             (C)2002 Markus Ewald -> License.txt  //
//  //
#include "Confix/ConfigFile.h"
#include <stdexcept>

#ifdef CONFIX_NAMESPACE
  using namespace CONFIX_NAMESPACE;
#endif CONFIX_NAMESPACE

// ####################################################################### //
// # CFX::CConfigFile::CConfigFile()                         Constructor # //
// ####################################################################### //
/** Initializes an instance of CConfigFile
*/
CConfigFile::CConfigFile(void) :
  m_pSections(new SectionVector()) {

  return;
}

// ####################################################################### //
// # CFX::CConfigFile::CConfigFile()                         Constructor # //
// ####################################################################### //
/** Initializes an instance of CConfigFile
*/
CConfigFile::CConfigFile(const std::string &sContents) :
  m_pSections(new SectionVector()) {
  setContents(sContents);

  return;
}

// ####################################################################### //
// # CFX::CConfigFile::~CConfigFile()                         Destructor # //
// ####################################################################### //
/** Destroys an instance of CConfigFile
*/
CConfigFile::~CConfigFile(void) {
  clearContents();
  delete m_pSections;

  return;
}

// ####################################################################### //
// # CFX::CConfigFile::clearContents()                                   # //
// ####################################################################### //
/** Clears the file's contents
*/
void CConfigFile::clearContents(void) {
  m_pSections->clear();

  return;
}

// ####################################################################### //
// # CFX::CConfigFile::getContents()                                     # //
// ####################################################################### //
/** Retrieves the configuration file contents for this variable

    @return The configuration file contents for the variable
*/
std::string CConfigFile::getContents(void) const { 
  std::string sContents;

  for(unsigned long lSection = 0; lSection < m_pSections->size(); lSection++)
    sContents += m_pSections->operator[](lSection).getContents();

  return sContents;
}

// ####################################################################### //
// # CFX::CConfigFile::setContents()                                     # //
// ####################################################################### //
/** Sets the configuration file contents for this variable

    @param  sContents  The configuration file contents for the variable
*/
std::string::size_type CConfigFile::setContents(const std::string &sContents) {
  std::string::size_type Pos = 0;

  clearContents();

  // Ensure that the string is not empty
  if(sContents.length() != 0) {
    while(Pos < sContents.length()) {
      CConfigSection Section;

      Pos = Section.setContents(sContents, Pos);

      if(m_pSections->size() >= m_pSections->capacity())
        m_pSections->reserve(m_pSections->size() + 16);

      m_pSections->push_back(Section);
    }
  }

  return Pos;
}

// ####################################################################### //
// # CFX::CConfigFile::getNumSections()                                  # //
// ####################################################################### //
/** Get number of variables in the config file

    @return The total number of variables
*/
unsigned long CConfigFile::getNumSections(void) const {
  return m_pSections->size();
}

// ####################################################################### //
// # CFX::CConfigFile::getSection()                                      # //
// ####################################################################### //
/** Retrieve a variable from the file

    @param  lIndex  Index of the variable to retrieve
    @return The variable's index
*/
CConfigSection &CConfigFile::getSection(unsigned long nIndex) {
  if(nIndex >= m_pSections->size())
    throw std::out_of_range("The specified index is out of range");

  return m_pSections->operator[](nIndex);
}

// ####################################################################### //
// # CFX::CConfigFile::getSection()                                      # //
// ####################################################################### //
/** Retrieve a variable from the file

    @param  sName  Name of the variable to retrieve
    @return The variable's index
*/
CConfigSection &CConfigFile::getSection(const std::string &sName) {
  SectionVector::iterator SectionEnd = m_pSections->end();
  for(SectionVector::iterator SectionIt = m_pSections->begin();
      SectionIt != SectionEnd;
      SectionIt++) {
    if(SectionIt->getName() == sName)
      return *SectionIt;
  }

  throw std::invalid_argument("A variable with the specified name could not be found");
}

// ####################################################################### //
// # CFX::CConfigFile::addSection()                                      # //
// ####################################################################### //
/** Add a variable to the file

    @param  pSection  Section to add
    @return The variable's index
*/
void CConfigFile::addSection(const CConfigSection &Section) {
  m_pSections->push_back(Section);

  return;
}

// ####################################################################### //
// # CFX::CConfigFile::removeSection()                                   # //
// ####################################################################### //
/** Remove a variable from the file

    @param  lIndex  Index of the variable to remove
*/
void CConfigFile::removeSection(unsigned long lIndex) {
  if(lIndex >= m_pSections->size())
    throw std::out_of_range("The specified index is out of range");

  m_pSections->erase(m_pSections->begin() + lIndex);

  return;
}

// ####################################################################### //
// # CFX::CConfigFile::removeSection()                                   # //
// ####################################################################### //
/** Remove a variable from the file

    @param  sName  Name of the variable to remove
*/
void CConfigFile::removeSection(const std::string &sName) {
  bool bFound = false;

  SectionVector::iterator SectionEnd = m_pSections->end();
  for(SectionVector::iterator SectionIt = m_pSections->begin();
      SectionIt != SectionEnd;
      SectionIt++) {
    if(SectionIt->getName() == sName) {
      m_pSections->erase(SectionIt);
      bFound = true;
    }
  }

  if(!bFound)
    throw std::invalid_argument("A variable with the specified name could not be found");
}

// ####################################################################### //
// # CFX::CConfigFile::getSections()                                     # //
// ####################################################################### //
/** Get a new variable enumerator

    @return A new variable enumerator
*/
CConfigFile::CSectionEnumerator CConfigFile::getSections(void) {
  return CSectionEnumerator(*m_pSections);
}

// ####################################################################### //
// # CFX::CConfigFile::CSectionEnumerator::CSectionEnumerator()          # Constructor //
// ####################################################################### //
/** Initializes an instance of CSectionEnumerator 

    @param  Sections  The variables which to enumerate
*/
CConfigFile::CSectionEnumerator::CSectionEnumerator(SectionVector &Sections) :
  m_Sections(Sections),
  m_SectionIt(Sections.end()),
  m_SectionEnd(m_SectionIt) {

  return;
}

// ####################################################################### //
// # CFX::CConfigFile::CSectionEnumerator::~CSectionEnumerator()         # Destructor //
// ####################################################################### //
/** Destroys an instance of CSectionEnumerator 
*/
CConfigFile::CSectionEnumerator::~CSectionEnumerator(void) {
  return;
}

// ####################################################################### //
// # CFX::CConfigFile::CSectionEnumerator::getSection()                  # //
// ####################################################################### //
/** Returns the current config variable

    @return The current config variable
*/
CConfigSection &CConfigFile::CSectionEnumerator::getSection(void) {
  if(m_SectionIt == m_SectionEnd)
    throw std::out_of_range("The enumerator is in the empty cycle position");

  return *m_SectionIt;
}

// ####################################################################### //
// # CFX::CConfigFile::CSectionEnumerator::toNext()                      # //
// ####################################################################### //
/** Advances to the next variable        

    @return True until the end is reached
*/
bool CConfigFile::CSectionEnumerator::toNext(void) { 
  if(m_SectionIt == m_SectionEnd)
    m_SectionIt = m_Sections.begin();
  else
    m_SectionIt++;

  if(m_SectionIt != m_SectionEnd)
    return true;
  else
    return false;
}
