//  //
//  ###  ##### #   #               -= Confix Library =-                  //
// #   # #     ## ## ConfigVariable.cpp - Configuration file variable    //
// #     ####   ###                                                      //
// #     #      ###  Representation of a configuration file variable     //
// #   # #     ## ##                                                     //
//  ###  #     #   # R1             (C)2002 Markus Ewald -> License.txt  //
//  //
#include "Confix/ConfigVariable.h"
#include <stdio.h> //!!TEMP
#include "../../Demo/TimeMonitor.h"

#ifdef CONFIX_NAMESPACE
  using namespace CONFIX_NAMESPACE;
#endif CONFIX_NAMESPACE

/// Convert integer to string
static std::string itos(long nValue);
/// Convert integer to string
static std::string htos(long nValue);
/// Convert double to string
static std::string ftos(double dValue, long nDecimals = 6);

/// Check whether the character is a letter
/** Returns true if the specified character is a letter, being either
    in range from a-z (upper or lower case) or the special character '_'

    @param  Char  Character to check
    @return True if the character is a letter
*/
static inline bool isletter(std::string::value_type Char) {
  return (((Char >= 'a') && (Char <= 'z')) || ((Char >= 'A') && (Char <= 'Z')) || (Char == '_'));
}

/// Check whether the character is numeric
/** Returns true if the specified character is a number, being
    in the range from 0-9.

    @param  Char  Character to check
    @return True if the character is a number
*/
static inline bool isnumeric(std::string::value_type Char) {
  return (((Char >= '1') && (Char <= '9')) || (Char == '0'));
}

/// Check whether the character is a hexadecimal number
/** Returns true if the specified character is a hexadecimal number,
    being in the range from 0-9 or a-z (upper or lower case)

    @param  Char  Character to check
    @return True if the character is a hexadecimal number
*/
static inline bool ishex(std::string::value_type Char) {
  return (((Char >= '1') && (Char <= '9')) || ((Char >= 'a') && (Char <= 'f')) || ((Char >= 'A') && (Char <= 'F'))  || (Char == '0'));
}

// ####################################################################### //
// # CFX::CConfigVariable::CConfigVariable()                 Constructor # //
// ####################################################################### //
/** Initializes an instance of CConfigVariable
*/
CConfigVariable::CConfigVariable(void) :
  m_psName(new std::string()),
  m_eType(TYPE::T_NONE),
  m_bChanged(false),
  m_psContents(NULL) {
  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::CConfigVariable()                 Constructor # //
// ####################################################################### //
/** Initializes an instance of CConfigVariable

    @param  sContents  Initial contents string
*/
CConfigVariable::CConfigVariable(const std::string &sContents) :
  m_psName(new std::string()),
  m_eType(TYPE::T_NONE),
  m_bChanged(false),
  m_psContents(NULL) {
  if(sContents[sContents.length() - 1] != '\n')
    setContents(sContents + '\n');
  else
    setContents(sContents);

  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::CConfigVariable()                 Constructor # //
// ####################################################################### //
/** Initializes an instance of CConfigVariable as copy of an
    existing instance
    
    @param  Variable  Variable to copy
*/
CConfigVariable::CConfigVariable(const CConfigVariable &Variable) :
  m_psName(new std::string(*Variable.m_psName)),
  m_eType(Variable.m_eType),
  m_bChanged(false),
  m_psContents(NULL),
  m_NameStart(Variable.m_NameStart),
  m_NameLength(Variable.m_NameLength),
  m_ValueStart(Variable.m_ValueStart),
  m_ValueLength(Variable.m_ValueLength) {

  if((m_eType == TYPE::T_STRING) || (m_eType == TYPE::T_STRING))
    m_psValue = new std::string(*Variable.m_psValue);

  if(Variable.m_psContents)
    m_psContents = new std::string(*Variable.m_psContents);

  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::~CConfigVariable()                 Destructor # //
// ####################################################################### //
/** Destroys an instance of CConfigVariable
*/
CConfigVariable::~CConfigVariable(void) {
  clearContents();
  delete m_psName;

  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::clearContents()                               # //
// ####################################################################### //
/** Clears the variable's contents
*/
void CConfigVariable::clearContents(void) {
  clearValue();  

  delete m_psContents;
  m_psContents = NULL;
  *m_psName = "";

  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::clearValue()                                  # //
// ####################################################################### //
/** Clears the variable's contents
*/
void CConfigVariable::clearValue(void) {
  if(m_eType == TYPE::T_STRING)
    delete m_psValue;
  
  m_eType = TYPE::T_NONE;

  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::getContents()                                 # //
// ####################################################################### //
/** Retrieves the configuration file contents for this variable

    @return The configuration file contents for the variable
*/
const std::string &CConfigVariable::getContents(void) const { 
  static std::string sEmpty;

  // If the variable hasn't changed, just return its current contents
  if(!m_bChanged) {
    if(m_psContents)
      return *m_psContents;
    else
      return sEmpty;

  // The variable has been changed since last time
  } else {

    // Has the contents string been created already ?
    if(m_psContents) {
      switch(m_eType) {
        case TYPE::T_BOOL: 
        case TYPE::T_LONG:
        case TYPE::T_HEX:
        case TYPE::T_DOUBLE:
        case TYPE::T_STRING: {
            *m_psContents = m_psContents->substr(0, m_NameStart) + *m_psName + m_psContents->substr(m_NameStart + m_NameLength);
            m_ValueStart += m_psName->length() - m_NameLength;
            m_NameLength = m_psName->length();

          // Place quotation marks around strings
          if(m_eType == TYPE::T_STRING) {
            *m_psContents = m_psContents->substr(0, m_ValueStart) + '"' + getString() + '"' + m_psContents->substr(m_ValueStart + m_ValueLength);
            m_ValueLength = getString().length() + 2;
          } else {
            *m_psContents = m_psContents->substr(0, m_ValueStart) + getString() + m_psContents->substr(m_ValueStart + m_ValueLength);
            m_ValueLength = getString().length();
          }

          m_bChanged = false;
          return *m_psContents;
        }

        default: {
          delete m_psContents;
          m_psContents = NULL;
          break;
        }
      }

      return sEmpty;
 
    // The contents string wasn't created yet
    } else {
      switch(m_eType) {
        case TYPE::T_BOOL: 
        case TYPE::T_LONG:
        case TYPE::T_HEX:
        case TYPE::T_DOUBLE:
        case TYPE::T_STRING: {
          // Place quotation marks around strings
          if(m_eType == TYPE::T_STRING) {
            m_psContents = new std::string(*m_psName + " = \"" + getString() + "\"\n");
            m_NameStart = 0;
            m_NameLength = m_psName->length();
            m_ValueStart = m_NameLength + 3;
            m_ValueLength = getString().length() + 2;
          } else {
            m_psContents = new std::string(*m_psName + " = " + getString() + "\n");
            m_NameStart = 0;
            m_NameLength = m_psName->length();
            m_ValueStart = m_NameLength + 3;
            m_ValueLength = getString().length();
          }

          m_bChanged = false;
          return *m_psContents;
        }

        default: {
          delete m_psContents;
          m_psContents = NULL;
          break;
        }
      }

      return sEmpty;
    }
  }
}

// ####################################################################### //
// # CFX::CConfigVariable::setContents()                                 # //
// ####################################################################### //
/** Sets the configuration file contents for this variable

    @param  sContents  The configuration file contents for the variable
*/
std::string::size_type CConfigVariable::setContents(const std::string &sContents,
                                                    std::string::size_type Start) {
  std::string::size_type Pos = Start;

  clearContents();

  // Ensure that the string is not empty
  if(sContents.length() != 0) {

    // Look for the first non-spacing character
    Pos = sContents.find_first_not_of(" \t", Pos);
    if(Pos != std::string::npos) {
      std::string::size_type LastNonSpacePos = std::string::npos;
      m_NameStart = Pos;

      // Scan the string for an assignment
      while(Pos < sContents.length()) {
        std::string::value_type Char = sContents[Pos];

        if(Char == '\n')
          break;
        else if((Char == '\r') && (Char == '\n'))
          break;

        // If a comment is initiated, stop parsing
        if((Char == '#') || (Char == ';')) {
          Pos = scanComment(sContents, Pos);
          break;
        }
        if((Char == '/') && (sContents[Pos + 1] == '/')) {
          Pos = scanComment(sContents, Pos);
          break;
        }

        // If an assignment was found, try to retrieve the variable
        if(Char == '=') {
          if(LastNonSpacePos != std::string::npos) {
            m_NameLength = LastNonSpacePos - m_NameStart + 1;
            *m_psName = sContents.substr(m_NameStart, m_NameLength);

            Pos = scanValue(sContents, Pos + 1);
          }

          break;
        }

        // Update position of the last non-spacing character encountered
        if((Char != ' ') && (Char != '\t'))
          LastNonSpacePos = Pos;

        Pos++;
      }
    }

    std::string::size_type EndPos = sContents.find_first_of('\n', Pos);
    if(EndPos == std::string::npos)
      Pos = sContents.length();
    else
      Pos = EndPos + 1;
  }

  m_NameStart -= Start;
  m_ValueStart -= Start;
  m_psContents = new std::string(sContents.substr(Start, Pos - Start));
  m_bChanged = false;

  return Pos;
}

// ####################################################################### //
// # CFX::CConfigVariable::scanValue()                                   # //
// ####################################################################### //
/** Scans the specified string for a value after an assignment

    @param  sString  String to scan
*/
std::string::size_type CConfigVariable::scanValue(const std::string &sString, std::string::size_type Start) {
  std::string::size_type Pos = Start;

  // Ensure that the string is not empty
  if(sString.length() > 0) {

    // Look for the first non-spacing character
    Pos = sString.find_first_not_of(" \t", Start);
    if(Pos != std::string::npos) {
      std::string::value_type Char = sString[Pos];

      // Try as number and fallback to string on invalid characters
      std::string::size_type Length = scanNumber(sString, Pos);
      if(Length == std::string::npos)
        Pos = scanString(sString, Pos);
      else
        Pos = Length;
    }
  }

  return Pos;
}

// ####################################################################### //
// # CFX::CConfigVariable::scanComment()                                 # //
// ####################################################################### //
/** Scans the specified string for a string assignment

    @param  sString  String to scan
*/
std::string::size_type CConfigVariable::scanComment(const std::string &sString, std::string::size_type Pos) {
  if(sString.length() > 0) {
    while(Pos < sString.length()) {
      if(sString[Pos] == '\n')
        break;

      Pos++;
    }
  }

  return Pos;
}

// ####################################################################### //
// # CFX::CConfigVariable::scanString()                                  # //
// ####################################################################### //
/** Scans the specified string for a string assignment

    @param  sString  String to scan
*/
std::string::size_type CConfigVariable::scanString(const std::string &sString, std::string::size_type Start) {
  std::string::size_type Pos = Start;

  // Ensure that the string is not empty
  if(sString.length() > 0) {

    // Look for the first non-spacing character
    Pos = sString.find_first_not_of(" \t", Pos);
    if(Pos != std::string::npos) {
      std::string::value_type Char = sString[Pos];

      // If it starts with a quote, scan it using extended mode
      if((Char == '"') || (Char == '\n') || 
         ((Char == '\r') && (sString[Pos + 1] == '\n'))) {
        while(Pos < sString.length()) {
          Char = sString[Pos];

          if((Char == '\r') && (sString[Pos + 1] == '\n')) {
            Pos++;
            Char = sString[Pos];
          }
          
          if(Char == '"')
            break;
          else if((Char != ' ') && (Char != '\t') && (Char != '\n'))
            return Pos;
          
          Pos++;
        }

        std::string sResult;

        m_ValueStart = Pos;
        std::string::size_type LastStringPos = Pos;

        Pos++;

        // Scan the string until a final closing quote is found
        while(Pos < sString.length()) {
          std::string::value_type Char = sString[Pos];

          // Found closing quote ?
          if(Char == '"') {
            bool bContinue = false;

            // Check if the string is reopened with another quote
            Pos++;
            LastStringPos = Pos;
            while(Pos < sString.length()) {
              Char = sString[Pos];
              Pos++;

              // New quote found, continue scanning at this location
              if(Char == '"') {
                bContinue = true;
                break;

              // Break on anything but spaces and newlines
              } else if((Char != ' ') && (Char != '\t') && (Char != '\n')) {
                if((Char != '\r') || (sString[Pos] != '\n'))
                  break;
              }/* else if(Char == '\n') {
                LastStringPos = Pos - 1;
              }*/
            }

            // Continue only if the string was reopened
            if(bContinue)
              continue;
            else
              break;
        
          // A character other than a quote was found
          } else {

            // Is this an escape sequence ?
            if(Char == '\\') {
              bool bValid = true;

              // Try to resolve the escape character
              switch(sString[Pos + 1]) {
                case 'n': Char = '\n'; break;
                case 't': Char = '\t'; break;
                case 'v': Char = '\v'; break;
                case 'b': Char = '\b'; break;
                case 'r': Char = '\r'; break;
                case 'f': Char = '\f'; break;
                case 'a': Char = '\a'; break;
                case '\\': Char = '\\'; break;
                case '?': Char = '?'; break;
                case '\'': Char = '\''; break;
                case '\"': Char = '\"'; break;
                case '0': Char = '\0'; break;
                case '\n': break;
                default: bValid = false; break;
              }

              // Skip the escape character only if the sequence was valid
              if(bValid)
                Pos++;
            }

            // Append the character to the result
            sResult += Char;
            LastStringPos = Pos;
          }

          Pos++;
        }

        m_eType = TYPE::T_STRING;
        m_ValueLength = LastStringPos - m_ValueStart;
        m_psValue = new std::string(sResult);
        Pos = LastStringPos;

      // Scan a simple string
      } else {
        std::string::size_type LastNonSpacePos = Pos;
        m_ValueStart = Pos;

        while(Pos < sString.length()) {
          std::string::value_type Char = sString[Pos];

          // If a comment is initiated, stop parsing
          if((Char == '#') || (Char == ';'))
            break;
          if((Char == '/') && (sString[Pos + 1] == '/'))
            break;

          // Simple strings end with newline characters
          if((Char == '\n') || ((Char == '\r') && (sString[Pos + 1] == '\n')))
            break;

          if((Char != ' ') && (Char != '\t'))
            LastNonSpacePos = Pos;

          Pos++;
        }

        m_eType = TYPE::T_STRING;
        m_ValueLength = LastNonSpacePos - m_ValueStart + 1;
        m_psValue = new std::string(sString.substr(m_ValueStart, m_ValueLength));
      }
    }
  }

  return Pos;
}

// ####################################################################### //
// # CFX::CConfigVariable::scanNumber()                                  # //
// ####################################################################### //
/** Scans the specified string for a number assignment

    @param  sString  String to scan
*/
std::string::size_type CConfigVariable::scanNumber(const std::string &sString, std::string::size_type Pos) {
  // Ensure that the string is not empty
  if(sString.length() > 0) {
    std::string::value_type Char = sString[Pos];
    bool bNegative = false;

    m_ValueStart = Pos;

    if((Char == '-') || (Char == '+')) {
      if(Char == '-')
        bNegative = true;

      Pos++;
      Char = sString[Pos];
    }

    // If it starts with a single quote, scan it as character constant
    if(Char == '\'') {
      Pos++;

      // Build the number by shifting and oring the current character
      long lValue = 0;
      while(Pos < sString.length()) {
        Char = sString[Pos];

        // Break if a closing single quote is found
        if(Char == '\'') {
          break;

        } else {
          // Is this an escape sequence ?
          if(Char == '\\') {
            bool bValid = true;

            // Try to resolve the escape character
            switch(sString[Pos + 1]) {
              case 'n': Char = '\n'; break;
              case 't': Char = '\t'; break;
              case 'v': Char = '\v'; break;
              case 'b': Char = '\b'; break;
              case 'r': Char = '\r'; break;
              case 'f': Char = '\f'; break;
              case 'a': Char = '\a'; break;
              case '\\': Char = '\\'; break;
              case '?': Char = '?'; break;
              case '\'': Char = '\''; break;
              case '\"': Char = '\"'; break;
              case '0': Char = '\0'; break;
              case '\n': break;
              default: bValid = false; break;
            }

            // Skip the escape character only if the sequence was valid
            if(bValid)
              Pos++;
          }

          lValue <<= (sizeof(std::string::value_type) * 8);
          lValue |= Char;
        }

        Pos++;
      }
      if(Pos < sString.length()) {
        Char = sString[Pos + 1];
        if((Char != ' ') && (Char != '\t') && (Char != '\n'))
          if((Char != '\r') || (sString[Pos + 1] != '\n'))
            return std::string::npos;
      }

      if(bNegative)
        lValue = -lValue;

      m_eType = TYPE::T_LONG;
      m_ValueLength = Pos - m_ValueStart;
      m_lValue = lValue;
    
    // No character constant initiated
    } else {
      long lValue = 0;

      // If it starts with 0x, interpret it as hexadecimal number
      if((Char == '0') && ((sString[Pos + 1] == 'x') || (sString[Pos + 1] == 'X'))) {
        Pos += 2;

        // Scan the string and build its hexadecimal value
        while(Pos < sString.length()) {
          Char = sString[Pos];

          bool bValid = true;
          switch(Char) {
            case '0': Char = 0; break;
            case '1': Char = 1; break;
            case '2': Char = 2; break;
            case '3': Char = 3; break;
            case '4': Char = 4; break;
            case '5': Char = 5; break;
            case '6': Char = 6; break;
            case '7': Char = 7; break;
            case '8': Char = 8; break;
            case '9': Char = 9; break;
            case 'a': case 'A': Char = 10; break;
            case 'b': case 'B': Char = 11; break;
            case 'c': case 'C': Char = 12; break;
            case 'd': case 'D': Char = 13; break;
            case 'e': case 'E': Char = 14; break;
            case 'f': case 'F': Char = 15; break;
            default: bValid = false; break;
          }
          if(bValid) {
            lValue <<= 4;
            lValue |= Char;
          } else {
            return std::string::npos;
          }

          Pos++;
        }

        m_eType = TYPE::T_HEX;
        m_ValueLength = Pos - m_ValueStart;
        m_lValue = lValue;

      // Read the number as integer or floating point value
      } else {
        std::string::size_type Start = Pos;

        bool bDecimals = false;
        while(Pos < sString.length()) {
          Char = sString[Pos];

          if(Char == '.') {
            if(bDecimals)
              return std::string::npos;
              
            bDecimals = true;
          } else if(!isnumeric(Char)) {
            return std::string::npos;
          }

          Pos++;
        }

        // Convert the number either as integer or as floating point value
        if(bDecimals) {
          m_eType = TYPE::T_DOUBLE;
          m_dValue = ::atof(((bNegative ? std::string("-") : std::string()) + sString.substr(Start, Pos - Start)).c_str());
        } else {
          m_eType = TYPE::T_LONG;
          m_lValue = ::atoi(((bNegative ? std::string("-") : std::string()) + sString.substr(Start, Pos - Start)).c_str());
        }
        m_ValueLength = Pos - m_ValueStart;
      }
    }
  }

  return Pos;
}

// ####################################################################### //
// # CFX::CConfigVariable::getName()                                     # //
// ####################################################################### //
/** Retrieves the name of the configuration variable

    @return The variable's name
*/
const std::string &CConfigVariable::getName(void) const {
  return *m_psName;
}

// ####################################################################### //
// # CFX::CConfigVariable::setName()                                     # //
// ####################################################################### //
/** Sets the name of the configuration variable

    @param  sName  New name for the variable
*/
void CConfigVariable::setName(const std::string &sName) {
  *m_psName = sName;
  m_bChanged = true;

  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::getType()                                     # //
// ####################################################################### //
/** Retrieves the type of the configuration variable

    @return The variable's type
*/
CConfigVariable::TYPE CConfigVariable::getType(void) const {
  return m_eType;
}

// ####################################################################### //
// # CFX::CConfigVariable::getBool()                                     # //
// ####################################################################### //
/** Retrieves the configuration value as boolean

    @return The variable's boolean representation
*/
bool CConfigVariable::getBool(void) const {
  switch(m_eType) {
    case TYPE::T_BOOL: return m_bValue;
    case TYPE::T_LONG: return m_lValue != 0;
    case TYPE::T_HEX: return m_lValue != 0;
    case TYPE::T_DOUBLE: return m_dValue != 0.0;
    case TYPE::T_STRING: return m_psValue->length() > 0;
    default: return false;
  }
}

// ####################################################################### //
// # CFX::CConfigVariable::getLong()                                     # //
// ####################################################################### //
/** Retrieves the configuration value as long

    @return The variable's long representation
*/
long CConfigVariable::getLong(void) const {
  switch(m_eType) {
    case TYPE::T_BOOL: return static_cast<long>(m_bValue);
    case TYPE::T_LONG: return m_lValue;
    case TYPE::T_HEX: return m_lValue;
    case TYPE::T_DOUBLE: return static_cast<long>(m_dValue + 0.5);
    case TYPE::T_STRING: return ::atoi(m_psValue->c_str());
    default: return 0;
  }
}

// ####################################################################### //
// # CFX::CConfigVariable::getDouble()                                   # //
// ####################################################################### //
/** Retrieves the configuration value as double

    @return The variable's double representation
*/
double CConfigVariable::getDouble(void) const {
  switch(m_eType) {
    case TYPE::T_BOOL: return static_cast<double>(m_bValue);
    case TYPE::T_LONG: return static_cast<double>(m_lValue);
    case TYPE::T_HEX: return static_cast<double>(m_lValue);
    case TYPE::T_DOUBLE: return m_dValue;
    case TYPE::T_STRING: return ::atof(m_psValue->c_str());
    default: return 0.0;
  }
}

// ####################################################################### //
// # CFX::CConfigVariable::getString()                                   # //
// ####################################################################### //
/** Retrieves the configuration value as string

    @return The variable's string representation
*/
std::string CConfigVariable::getString(void) const {
  switch(m_eType) {
    case TYPE::T_BOOL: return m_bValue ? "1" : "0";
    case TYPE::T_LONG: return itos(m_lValue);
    case TYPE::T_HEX: return htos(m_lValue);
    case TYPE::T_DOUBLE: return ftos(m_dValue);
    case TYPE::T_STRING: return *m_psValue;
    default: return "";
  }
}

// ####################################################################### //
// # CFX::CConfigVariable::operator =()                                  # // 
// ####################################################################### //
/** Assigns a value to the CConfigVariable. If the assigned value is of type
    Blob or Object, AddRef() will automatically be called on it.

    @param  Value  Value to assign
    @return The assigned value
*/
CConfigVariable &CConfigVariable::operator =(const CConfigVariable &Value) {
  clearContents();

  m_eType = Value.m_eType;
  switch(m_eType) {
    case TYPE::T_BOOL: m_bValue = Value.m_bValue; break;
    case TYPE::T_LONG: case TYPE::T_HEX: m_lValue = Value.m_lValue; break;
    case TYPE::T_DOUBLE: m_dValue = Value.m_dValue; break;
    case TYPE::T_STRING: case TYPE::T_COMMENT: m_psValue = new std::string(*Value.m_psValue); break;
  }
  m_bChanged = true;

  return *this;
}

// ####################################################################### //
// # CFX::CConfigVariable::setBool()                                     # // 
// ####################################################################### //
/** Assigns a boolean value to the CConfigVariable

    @param  bValue  Boolean value
    @return The assigned value
*/
void CConfigVariable::setBool(bool bValue) {
  clearValue();

  m_eType = TYPE::T_BOOL;
  m_bValue = bValue;
  m_bChanged = true;

  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::setLong()                                     # // 
// ####################################################################### //
/** Assigns an integer value to the CConfigVariable

    @param  nValue  Integer value
    @return The assigned value
*/
void CConfigVariable::setLong(long nValue) {
  clearValue();

  m_eType = TYPE::T_LONG;
  m_lValue = nValue;
  m_bChanged = true;

  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::setDouble()                                   # // 
// ####################################################################### //
/** Assigns a double value to the CConfigVariable

    @param  dValue  Double value
    @return The assigned value
*/
void CConfigVariable::setDouble(double dValue) {
  clearValue();

  m_eType = TYPE::T_DOUBLE;
  m_dValue = dValue;
  m_bChanged = true;

  return;
}

// ####################################################################### //
// # CFX::CConfigVariable::setString()                                   # // 
// ####################################################################### //
/** Assigns a string value to the CConfigVariable

    @param  sString  String value
    @return The assigned value
*/
void CConfigVariable::setString(const std::string &sValue) {
  clearValue();

  m_eType = TYPE::T_STRING;
  m_psValue = new std::string(sValue);
  m_bChanged = true;

  return;
}

// ####################################################################### //
// # itos()                                                              # // 
// ####################################################################### //
/** Converts an integer into a string

    @param  nValue  Integer value to convert
    @return The string representation of the integer
*/
std::string itos(long nValue) {
  char pszValue[11];
  ::itoa(nValue, pszValue, 10);

  return pszValue; // Implicitely constructs an std::string
}

// ####################################################################### //
// # htos()                                                              # // 
// ####################################################################### //
/** Converts an integer into a string

    @param  nValue  Integer value to convert
    @return The string representation of the integer
*/
std::string htos(long nValue) {
  char pszValue[11];
  pszValue[0] = '0';
  pszValue[1] = 'x';
  ::itoa(nValue, &pszValue[2], 16);

  return pszValue; // Implicitely constructs an std::string
}

// ####################################################################### //
// # ftos()                                                              # // 
// ####################################################################### //
/** Converts a double into a string

    @param  dValue     Double value to convert
    @param  nDecimals  Number of decimal places to return
    @return The string representation of the double
*/
std::string ftos(double dValue, long nDecimals) {
  int nDec, nSign;

  std::string sTemp = ::fcvt(dValue, nDecimals, &nDec, &nSign);

  return (((nDec == 0) ? (nSign ? std::string("-0") : std::string("0"))
                       : (nSign ? std::string("-") : std::string())) +
          sTemp.substr(0, nDec) + "." + sTemp.substr(nDec)).c_str();
}
